SystemOrganization addCategory: #CSV!

TestCase subclass: #CSVParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CSV'!

!CSVParserTest methodsFor: 'utils' stamp: 'LaurentLaffont 8/26/2011 11:04'!
removeBlanks: aString
	^ (aString copyWithoutAll: String crlf, Character space asString, Character tab asString )! !

!CSVParserTest methodsFor: 'tests' stamp: 'Damir 9/2/2011 21:21'!
testComaIsDefaultDelimiter
	| csv csvParser |
	csv := 'Luke,Skywalker,"Young Jedi"'.
	
	csvParser := CSVParser onString: csv.
	
	self 
		assert: {'Luke'. 'Skywalker'. 'Young Jedi'} asOrderedCollection 
		equals: csvParser rows first.! !

!CSVParserTest methodsFor: 'tests' stamp: 'LaurentLaffont 8/26/2011 11:07'!
testToJSON
	|csv parser actualJSON expectedJSON|
	csv := String streamContents: [:aStream|
			aStream 
				nextPutAll: 'firsname, lastname, description'; cr;
				nextPutAll: 'Luke,Skywalker,"Young Jedi"'; cr;
				nextPutAll: 'Obiwan, Kenobi,"Jedi Master"'.
	].

	expectedJSON := '[{fname: "Luke", lname: "Skywalker", activity: "Young Jedi"},
						{fname: "Obiwan", lname: "Kenobi", activity: "Jedi Master"}]'.

	parser := CSVParser  onString: csv.
	actualJSON := parser asJSONWithHeader: #(fname lname activity).
	self 
		assert:	(self removeBlanks: expectedJSON) 
		equals:  (self removeBlanks: actualJSON)
		! !

!CSVParserTest methodsFor: 'tests' stamp: 'Damir 9/2/2011 21:16'!
testWithSemiColonDelimiter
	| csv csvParser |
	csv := 'Luke;Skywalker;"Young Jedi"'.
	
	csvParser := CSVParser onString: csv.
	csvParser useDelimiter: $;.
	
	self 
		assert: {'Luke'. 'Skywalker'. 'Young Jedi'} asOrderedCollection 
		equals: csvParser rows first.! !

Object subclass: #CSVParser
	instanceVariableNames: 'stream delimiter rowClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CSV'!

!CSVParser commentStamp: '<historical>' prior: 0!
A simple parser for comma-separated-value files.
http://www.squeaksource.com/CSV.html

Example:

rows := CSVParser parse: (FileStream readOnlyFileNamed: '/path/to/file.csv')!

!CSVParser class methodsFor: 'initialize-release' stamp: 'Damir 9/2/2011 21:25'!
on: aStream
	^ self new initializeWithStream: aStream! !

!CSVParser class methodsFor: 'initialize-release' stamp: 'LaurentLaffont 8/26/2011 10:48'!
onString: aString
	^ self on: aString readStream! !

!CSVParser class methodsFor: 'initialize-release' stamp: 'LaurentLaffont 8/26/2011 10:48'!
parse: aStream
	^ (self on: aStream) rows! !

!CSVParser class methodsFor: 'initialize-release' stamp: 'avi 7/4/2005 22:08'!
parseString: aString
	^ self parse: aString readStream! !

!CSVParser methodsFor: 'converting' stamp: 'LaurentLaffont 8/26/2011 10:53'!
asJSONWithHeader: aCollectionOfString
	|rows|
	rows := self rows copyWithoutIndex: 1.

	^  String streamContents: [:aStream|
		aStream nextPutAll: '['; cr.
		
		rows do: [:row |
			aStream nextPutAll: ' {'.
			aCollectionOfString	do: [:property|
				aStream nextPutAll: ('	{1}: "{2}"' format: {property asString. row removeAt:1})
			]
			separatedBy: [aStream nextPutAll: ','; cr].
		aStream nextPutAll: ' }'.
		]
		separatedBy: [aStream nextPutAll: ','; cr].
		
		aStream nextPutAll: ']'.		
	].  ! !

!CSVParser methodsFor: 'testing' stamp: 'TomRake 11/16/2011 22:04'!
atEndOfLine
	^ stream atEnd or: [stream peek = Character cr or: [stream peek = Character lf]]! !

!CSVParser methodsFor: 'initialization' stamp: 'Damir 9/2/2011 21:20'!
initialize
	delimiter := $,! !

!CSVParser methodsFor: 'initialization' stamp: 'LaurentLaffont 8/26/2011 09:53'!
initializeWithStream: aStream
	stream := aStream! !

!CSVParser methodsFor: 'parsing' stamp: 'LaurentLaffont 8/26/2011 09:55'!
nextInLine
	| next |
	next := stream next.
	(next = Character cr or: [next = Character lf])
		ifTrue:	[stream skip: -1. next := nil].
	^ next! !

!CSVParser methodsFor: 'parsing' stamp: 'avi 9/19/2005 23:48'!
nextQuotedValue
	^ String streamContents:
		[:s |
		s nextPutAll: (stream upTo: $").
		self nextInLine = $" ifTrue:
			[s nextPut: $".
			s nextPutAll: self nextQuotedValue]]! !

!CSVParser methodsFor: 'parsing' stamp: 'Kaspersterbye 11/29/2011 08:51'!
nextRow
	| row |
	row := self rowClass new.
	stream skipSeparators.
	[self atEndOfLine]
		whileFalse: [row add: self nextValue].
	stream skip: -1.
	stream next = $,
		ifTrue: [row add: ''].
	^ row! !

!CSVParser methodsFor: 'parsing' stamp: 'Damir 9/2/2011 21:25'!
nextValue
	| next |
	

	
	stream peek = $" ifTrue: [stream next. ^ self nextQuotedValue].
	^ String streamContents:
		[:s |
		[(next := self nextInLine) notNil and: [next ~= delimiter]]
			whileTrue: [s nextPut: next]]! !

!CSVParser methodsFor: 'accessing' stamp: 'JohnnyT 4/13/2012 17:07'!
rowClass
	^ rowClass ifNil: [rowClass := OrderedCollection]! !

!CSVParser methodsFor: 'accessing' stamp: 'avi 7/4/2005 22:08'!
rows
	^ Array streamContents: [:s | self rowsDo: [:ea | s nextPut: ea]]! !

!CSVParser methodsFor: 'enumerating' stamp: 'LaurentLaffont 8/26/2011 09:55'!
rowsDo: aBlock
	| row |
	[stream atEnd] whileFalse:
		[row := self nextRow.
		row isEmpty ifFalse: [aBlock value: row]]! !

!CSVParser methodsFor: 'accessing' stamp: 'Damir 9/2/2011 21:17'!
useDelimiter: aCharacter 
	delimiter := aCharacter.! !

!CSVParser methodsFor: 'accessing' stamp: 'JohnnyT 4/13/2012 17:16'!
useRowClass: aClass
	rowClass := aClass! !
